@@ -35,6 +35,7 @@
 #include <signal.h>
 #include <getopt.h>
 #include <time.h>
+#include <locale.h>
 
 #include "overlay.h"
 #include "chart.h"
@@ -312,8 +313,9 @@ static void show_gpu_perf(struct overlay_context *ctx, struct overlay_gpu_perf *
 	struct gpu_perf_comm *comm, **prev;
 	const char *ring_name[] = {
 		"R",
-		"V",
 		"B",
+		"V0",
+		"V1",
 	};
 	double range[2];
 	char buf[1024];
@@ -362,6 +364,12 @@ static void show_gpu_perf(struct overlay_context *ctx, struct overlay_gpu_perf *
 	for (comm = gp->gpu_perf.comm; comm; comm = comm->next) {
 		int total;
 
+		if (comm->name[0] == '\0')
+			continue;
+
+		if (strncmp(comm->name, "kworker", 7) == 0)
+			continue;
+
 		if (comm->user_data == NULL) {
 			comm->user_data = malloc(sizeof(struct chart));
 			if (comm->user_data == NULL)
@@ -387,11 +395,18 @@ static void show_gpu_perf(struct overlay_context *ctx, struct overlay_gpu_perf *
 	}
 
 	range[0] = range[1] = 0;
-	for (comm = gp->gpu_perf.comm; comm; comm = comm->next)
+	for (comm = gp->gpu_perf.comm; comm; comm = comm->next) {
+		if (comm->user_data == NULL)
+			continue;
+
 		chart_get_range(comm->user_data, range);
+	}
 
 	y2 = y1 = y;
 	for (comm = gp->gpu_perf.comm; comm; comm = comm->next) {
+		if (comm->user_data == NULL)
+			continue;
+
 		chart_set_range(comm->user_data, range[0], range[1]);
 		chart_draw(comm->user_data, ctx->cr);
 		y2 += 14;
@@ -414,11 +429,11 @@ static void show_gpu_perf(struct overlay_context *ctx, struct overlay_gpu_perf *
 	for (prev = &gp->gpu_perf.comm; (comm = *prev) != NULL; ) {
 		int need_comma = 0, len;
 
-		if (comm->name[0] == '\0')
+		if (comm->user_data == NULL)
 			goto skip_comm;
 
 		len = sprintf(buf, "%s:", comm->name);
-		for (n = 0; n < 3; n++) {
+		for (n = 0; n < sizeof(ring_name)/sizeof(ring_name[0]); n++) {
 			if (comm->nr_requests[n] == 0)
 				continue;
 			len += sprintf(buf + len, "%s %d%s", need_comma ? "," : "", comm->nr_requests[n], ring_name[n]);
@@ -467,8 +482,9 @@ static void show_gpu_perf(struct overlay_context *ctx, struct overlay_gpu_perf *
 
 skip_comm:
 		memset(comm->nr_requests, 0, sizeof(comm->nr_requests));
-		if (comm->show < ctx->time - IDLE_TIME ||
-		    strcmp(comm->name, get_comm(comm->pid, buf, sizeof(buf)))) {
+		if (!comm->active &&
+		    (comm->show < ctx->time - IDLE_TIME ||
+		     strcmp(comm->name, get_comm(comm->pid, buf, sizeof(buf))))) {
 			*prev = comm->next;
 			if (comm->user_data) {
 				chart_fini(comm->user_data);
@@ -724,7 +740,7 @@ static void init_gem_objects(struct overlay_context *ctx,
 static void show_gem_objects(struct overlay_context *ctx, struct overlay_gem_objects *go)
 {
 	struct gem_objects_comm *comm;
-	char buf[160];
+	char buf[310];
 	cairo_pattern_t *linear;
 	int x, y, y1, y2;
 
@@ -816,6 +832,17 @@ static void overlay_snapshot(struct overlay_context *ctx)
 	cairo_surface_write_to_png(ctx->surface, buf);
 }
 
+static void usage(const char *progname)
+{
+	printf("intel-gpu-overlay -- realtime display of GPU statistics\n");
+	printf("Usage: %s [options]\n", progname);
+	printf("\t--config|-c <string> | <filename>\t\t\tSpecify an ini-style configuration string or file\n");
+	printf("\t--geometry|-G <width>x<height>+<x-offset>+<y-offset>\tExact window placement and size\n");
+	printf("\t--position|-P (top|middle|bottom)-(left|centre|right)\tPlace the window in a particular corner\n");
+	printf("\t--size|-S <width>x<height> | <scale>%%\t\t\tWindow size\n");
+	printf("\t--help|-h\t\t\t\t\t\tThis help message\n");
+}
+
 int main(int argc, char **argv)
 {
 	static struct option long_options[] = {
@@ -823,6 +850,7 @@ int main(int argc, char **argv)
 		{"geometry", 1, 0, 'G'},
 		{"position", 1, 0, 'P'},
 		{"size", 1, 0, 'S'},
+		{"help", 0, 0, 'h'},
 		{NULL, 0, 0, 0,}
 	};
 	struct overlay_context ctx;
@@ -831,10 +859,11 @@ int main(int argc, char **argv)
 	int daemonize = 1, renice = 0;
 	int i;
 
+	setlocale(LC_ALL, "");
 	config_init(&config);
 
 	opterr = 0;
-	while ((i = getopt_long(argc, argv, "c:fn?", long_options, &index)) != -1) {
+	while ((i = getopt_long(argc, argv, "c:G:fhn?", long_options, &index)) != -1) {
 		switch (i) {
 		case 'c':
 			config_parse_string(&config, optarg);
@@ -856,6 +885,9 @@ int main(int argc, char **argv)
 			if (optarg)
 				renice = atoi(optarg);
 			break;
+		case 'h':
+			usage(argv[0]);
+			return 0;
 		}
 	}
 
@@ -874,13 +906,13 @@ int main(int argc, char **argv)
 	if (ctx.surface == NULL)
 		ctx.surface = kms_overlay_create(&config, &ctx.width, &ctx.height);
 	if (ctx.surface == NULL)
-		return ENOMEM;
+		return ENXIO;
 
 	if (daemonize && daemon(0, 0))
 		return EINVAL;
 
-	if (renice)
-		nice(renice);
+	if (renice && (nice(renice) == -1))
+		fprintf(stderr, "Could not renice: %s\n", strerror(errno));
 
 	signal(SIGUSR1, signal_snapshot);
 
