@@ -30,6 +30,7 @@
  * Testcase: run a couple of big batches to force the unbind on misalignment code.
  */
 
+#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -44,10 +45,6 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "igt_aux.h"
 
 IGT_TEST_DESCRIPTION("Run a couple of big batches to force the unbind on"
 		     " misalignment code.");
@@ -56,7 +53,8 @@ IGT_TEST_DESCRIPTION("Run a couple of big batches to force the unbind on"
 #define WIDTH 1024
 
 static void
-copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo, int alignment, int error)
+copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo,
+     uint64_t n_bo, uint64_t alignment, int error)
 {
 	uint32_t batch[12];
 	struct drm_i915_gem_relocation_entry reloc[2];
@@ -110,9 +108,9 @@ copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo, int alignme
 	}
 	obj[n].handle = handle;
 	obj[n].relocation_count = 2;
-	obj[n].relocs_ptr = (uintptr_t)reloc;
+	obj[n].relocs_ptr = to_user_pointer(reloc);
 
-	exec.buffers_ptr = (uintptr_t)obj;
+	exec.buffers_ptr = to_user_pointer(obj);
 	exec.buffer_count = n_bo + 1;
 	exec.batch_start_offset = 0;
 	exec.batch_len = i * 4;
@@ -126,16 +124,17 @@ copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo, int alignme
 	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
 	if (ret)
 		ret = errno;
-	igt_assert(ret == error);
+	igt_assert_eq(ret, error);
 
 	gem_close(fd, handle);
 	free(obj);
 }
 
-static void minor_evictions(int fd, int size, int count)
+static void minor_evictions(int fd, uint64_t size, uint64_t count)
 {
 	uint32_t *bo, *sel;
-	int n, m, alignment, pass, fail;
+	uint64_t n, m, alignment;
+	int pass, fail;
 
 	intel_require_memory(2 * count, size, CHECK_RAM);
 
@@ -162,9 +161,10 @@ static void minor_evictions(int fd, int size, int count)
 	free(bo);
 }
 
-static void major_evictions(int fd, int size, int count)
+static void major_evictions(int fd, uint64_t size, uint64_t count)
 {
-	int n, m, loop, alignment, max;
+	uint64_t n, m, alignment, max;
+	int loop;
 	uint32_t *bo;
 
 	intel_require_memory(count, size, CHECK_RAM);
@@ -188,42 +188,82 @@ static void major_evictions(int fd, int size, int count)
 	free(bo);
 }
 
-int fd;
+#define MAX_32b ((1ull << 32) - 4096)
 
 igt_main
 {
-	int size, count;
+	uint64_t size, count;
+	int fd = -1;
 
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
+		igt_fork_hang_detector(fd);
 	}
 
 	igt_subtest("minor-normal") {
 		size = 1024 * 1024;
-		count = 3*gem_aperture_size(fd) / size / 4;
+		count = gem_aperture_size(fd);
+		if (count >> 32)
+			count = MAX_32b;
+		count = 3 * count / size / 4;
 		minor_evictions(fd, size, count);
 	}
 
 	igt_subtest("major-normal") {
-		size = 3*gem_aperture_size(fd) / 4;
+		size = gem_aperture_size(fd);
+		if (size >> 32)
+			size = MAX_32b;
+		size = 3 * size / 4;
 		count = 4;
 		major_evictions(fd, size, count);
 	}
 
+	igt_fixture {
+		igt_stop_hang_detector();
+	}
+
 	igt_fork_signal_helper();
 	igt_subtest("minor-interruptible") {
 		size = 1024 * 1024;
-		count = 3*gem_aperture_size(fd) / size / 4;
+		count = gem_aperture_size(fd);
+		if (count >> 32)
+			count = MAX_32b;
+		count = 3 * count / size / 4;
 		minor_evictions(fd, size, count);
 	}
 
 	igt_subtest("major-interruptible") {
-		size = 3*gem_aperture_size(fd) / 4;
+		size = gem_aperture_size(fd);
+		if (size >> 32)
+			size = MAX_32b;
+		size = 3 * size / 4;
+		count = 4;
+		major_evictions(fd, size, count);
+	}
+
+	igt_subtest("minor-hang") {
+		igt_fork_hang_helper();
+		size = 1024 * 1024;
+		count = gem_aperture_size(fd);
+		if (count >> 32)
+			count = MAX_32b;
+		count = 3 * count / size / 4;
+		minor_evictions(fd, size, count);
+	}
+
+	igt_subtest("major-hang") {
+		size = gem_aperture_size(fd);
+		if (size >> 32)
+			size = MAX_32b;
+		size = 3 * size / 4;
 		count = 4;
 		major_evictions(fd, size, count);
 	}
 	igt_stop_signal_helper();
 
-	igt_fixture
+	igt_fixture {
+		igt_stop_hang_helper();
 		close(fd);
+	}
 }
