@@ -20,29 +20,19 @@
  * Authors:
  *    Imre Deak <imre.deak@intel.com>
  */
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
+#include "igt.h"
 #include <cairo.h>
 #include <errno.h>
 #include <stdint.h>
 #include <unistd.h>
 #include <sys/time.h>
 
-#include "drmtest.h"
 #include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_io.h"
-#include "intel_chipset.h"
-#include "igt_kms.h"
-#include "ioctl_wrappers.h"
 
 drmModeRes *resources;
 int drm_fd;
-static drm_intel_bufmgr *bufmgr;
-struct intel_batchbuffer *batch;
-uint32_t devid;
 
 enum test_flags {
 	TEST_DIRECT_RENDER	= 0x01,
@@ -75,6 +65,16 @@ static void gpu_blit(struct igt_fb *dst_fb, struct igt_fb *src_fb)
 	drm_intel_bo *dst_bo;
 	drm_intel_bo *src_bo;
 	int bpp;
+	static drm_intel_bufmgr *bufmgr;
+	struct intel_batchbuffer *batch;
+	uint32_t devid;
+
+	igt_require_intel(drm_fd);
+	igt_require_gem(drm_fd);
+
+	bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
+	devid = intel_get_drm_devid(drm_fd);
+	batch = intel_batchbuffer_alloc(bufmgr, devid);
 
 	igt_assert(dst_fb->drm_format == src_fb->drm_format);
 	igt_assert(src_fb->drm_format == DRM_FORMAT_RGB565 ||
@@ -112,12 +112,12 @@ static int test_format(const char *test_name,
 
 	ret = asprintf(&mode_format_str, "%s @ %dHz / %s",
 		 mode->name, mode->vrefresh, igt_format_str(format));
-	igt_assert(ret > 0);
+	igt_assert_lt(0, ret);
 	ret = asprintf(&cconf_str, "pipe %s, encoder %s, connector %s",
 		       kmstest_pipe_name(cconf->pipe),
 		       kmstest_encoder_type_str(cconf->encoder->encoder_type),
 		       kmstest_connector_type_str(cconf->connector->connector_type));
-	igt_assert(ret > 0);
+	igt_assert_lt(0, ret);
 
 	igt_info("Beginning test %s with %s on %s\n",
 		 test_name, mode_format_str, cconf_str);
@@ -125,12 +125,12 @@ static int test_format(const char *test_name,
 	width = mode->hdisplay;
 	height = mode->vdisplay;
 
-	if (!igt_create_fb(drm_fd, width, height, format, I915_TILING_NONE,
-			   &fb[0]))
+	if (!igt_create_fb(drm_fd, width, height, format,
+			   LOCAL_DRM_FORMAT_MOD_NONE, &fb[0]))
 		goto err1;
 
-	if (!igt_create_fb(drm_fd, width, height, format, I915_TILING_NONE,
-			   &fb[1]))
+	if (!igt_create_fb(drm_fd, width, height, format,
+			   LOCAL_DRM_FORMAT_MOD_NONE,	&fb[1]))
 		goto err2;
 
 	if (drmModeSetCrtc(drm_fd, cconf->crtc->crtc_id, fb[0].fb_id,
@@ -178,9 +178,10 @@ static void test_connector(const char *test_name,
 	int format_count;
 	int i;
 
-	igt_get_all_formats(&formats, &format_count);
+	igt_get_all_cairo_formats(&formats, &format_count);
 	for (i = 0; i < format_count; i++) {
-		if (intel_gen(intel_get_drm_devid(drm_fd)) < 4
+		if (is_i915_device(drm_fd)
+		    && intel_gen(intel_get_drm_devid(drm_fd)) < 4
 		    && formats[i] == DRM_FORMAT_XRGB2101010) {
 			igt_info("gen2/3 don't support 10bpc, skipping\n");
 			continue;
@@ -237,11 +238,7 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		drm_fd = drm_open_any_master();
-
-		bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
-		devid = intel_get_drm_devid(drm_fd);
-		batch = intel_batchbuffer_alloc(bufmgr, devid);
+		drm_fd = drm_open_driver_master(DRIVER_ANY);
 
 		kmstest_set_vt_graphics_mode();
 	}
