@@ -21,18 +21,18 @@
  * IN THE SOFTWARE.
  */
 
+#include "igt.h"
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 
+#include <sys/stat.h>
+
 #include "testdisplay.h"
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
-#include "igt_core.h"
 
-#if HAVE_UDEV
+#ifdef HAVE_UDEV
 #include <libudev.h>
 static struct udev_monitor *uevent_monitor;
 static struct udev *udev;
@@ -57,7 +57,7 @@ static gboolean hotplug_event(GIOChannel *source, GIOCondition condition,
 
 	if (memcmp(&s.st_rdev, &udev_devnum, sizeof(dev_t)) == 0 &&
 	    hotplug && atoi(hotplug) == 1)
-		update_display();
+		update_display(true);
 
 	udev_device_unref(dev);
 out:
